// Script to double check that all the json keys in
//  the english master translation exists in other language files

const fs = require('fs')

const MASTER_LANGUAGE_FILE = "en_US.json"


const banner = `
 ___   ___   _     _    _____  ___   ___   _    __
| |_) / / \\ | |   \\ \\_/  | |  / / \\ | |_) | |  / /\\
|_|   \\_\\_/ |_|__  |_|   |_|  \\_\\_/ |_|   |_| /_/--\\

`
console.log(banner)

console.log("Looking for all .json files in the current directory")
fs.readdir("./", (err, files) => {
    if (err) throw err

    const jsonFileName = name => name.match(/\.json$/i)
    const withoutMaster = name => name != MASTER_LANGUAGE_FILE
    const jsonFiles = files
        .filter(jsonFileName)
        .filter(withoutMaster)

    const hasMaster = files.includes(MASTER_LANGUAGE_FILE)
    if (!hasMaster) {
        console.error(`ERROR: Couldn't find ${MASTER_LANGUAGE_FILE} (the master language file)`)
        process.exit(1)
    }

    loadFile(MASTER_LANGUAGE_FILE)
        .then(file => {
            if (file.parseError) {
                console.error(`FATAL ERROR: Master language file isn't valid JSON`)
                console.error(file.parseError)
                process.exit(1)
            }
            return file
        })
        .then(master =>
            loadFiles(jsonFiles)
                .then(files => ({master, files}))
        )
        .then(({master, files}) => {
            compare(master, master)

            files.forEach(file => {
                compare(master, file)
            })
        })
})

function loadFile(name) {
    return new Promise(resolve => {
        console.log(`Loading file "${name}"`)
        fs.readFile(name, {encoding: 'utf-8'}, (err, text) => {
            if (err) throw err

            let json = null
            let parseError = null
            try {
                json = JSON.parse(text)
            }
            catch (e) {
                parseError = e.message
            }

            resolve({name, text, json, parseError});
       });
    })
}

function loadFiles(names) {
    return Promise.all(
        names.map(loadFile)
    )
}

function diffKeys(a, b) {
    const missing = []
    Object.keys(a).forEach(key => {
        if (!b.hasOwnProperty(key))
            missing.push(key)
    })
    return missing
}

function compare(master, translation) {
    let summary = []
    let hasProblems = false

    // Check parse error
    if (translation.parseError) {
        hasProblems = true
        summary.push(`Invalid JSON: ${translation.parseError}`)
    }
    else {
        const lines = translation.text.split("\n").length
        const keys = Object.keys(translation.json).length
        summary.push(`${lines} lines and ${keys} keys`)
        summary.push("")
    }


    // Check keys
    if (!hasProblems) {
        const missingKeys = diffKeys(master.json, translation.json)
        if (missingKeys.length) {
            hasProblems = true
            summary.push(`  Missing ${missingKeys.length} keys:`)
            summary.push(`    ${missingKeys.join("\n    ")}`)
            summary.push("")
        }

        const extraKeys = diffKeys(translation.json, master.json)
        if (extraKeys.length) {
            hasProblems = true
            summary.push(`  Found ${extraKeys.length} extra keys:`)
            summary.push(`    ${extraKeys.join("\n    ")}`)
            summary.push("")
        }
    }

    console.log(`\n*  ${translation.name} \t\t ${hasProblems ? "❌" : "👍"}`)
    console.log(`${summary.join("\n")}`)
}

