const fs = require('fs')

const MASTER_LANGUAGE_FILE = "en_US.json"


const banner = `

Fixing keys

`
console.log(banner)

console.log("Looking for all .json files in the current directory")
fs.readdir("./", (err, files) => {
    if (err) throw err

    const jsonFileName = name => name.match(/\.json$/i)
    const withoutMaster = name => name != MASTER_LANGUAGE_FILE
    const jsonFiles = files
        .filter(jsonFileName)
        .filter(withoutMaster)

    const hasMaster = files.includes(MASTER_LANGUAGE_FILE)
    if (!hasMaster) {
        console.error(`ERROR: Couldn't find ${MASTER_LANGUAGE_FILE} (the master language file)`)
        process.exit(1)
    }

    loadFile(MASTER_LANGUAGE_FILE)
        .then(file => {
            if (file.parseError) {
                console.error(`FATAL ERROR: Master language file isn't valid JSON`)
                console.error(file.parseError)
                process.exit(1)
            }
            return file
        })
        .then(master =>
            loadFiles(jsonFiles)
                .then(files => ({master, files}))
        )
        .then(({master, files}) => {
            files.forEach(file => {
                rebuild(master, file)
            })
        })
})

function loadFile(name) {
    return new Promise(resolve => {
        console.log(`Loading file "${name}"`)
        fs.readFile(name, {encoding: 'utf8'}, (err, text) => {
            if (err) throw err

            let json = null
            let parseError = null
            try {
                json = JSON.parse(text)
            }
            catch (e) {
                parseError = e.message
            }

            resolve({name, text, json, parseError});
       });
    })
}

function loadFiles(names) {
    return Promise.all(
        names.map(loadFile)
    )
}

function findLastIndex(arr, callback) {
    const copy = arr.concat()
    copy.reverse()
    return copy.findIndex(callback)
}

function rebuild(master, translation) {
    console.log(`\n* Rebuilding ${translation.name}`)

    // Check parse error
    if (translation.parseError) {
        console.log("Invalid JSON, can't fix")
        return;
    }

    const orgTransJSON = {...translation.json}
    const masterLines = master.text.split("\n")
    const masterLength = masterLines.length

    //const RE_EMPTY_LINE = /^\s*?[{|}]?\s*?$/
    const RE_FIND_KEY = /\s*?"(.*?)":./i
    //const isEmpty = line => line.match(RE_EMPTY_LINE)
    const getKey = line => {
        const found = line.match(RE_FIND_KEY)
        return found ? found[1] : null
    }

    const numEmptyEndLines = findLastIndex(masterLines, getKey)
    const newTranslation = []

    masterLines.forEach((mLine, mIndex) => {
        const key = getKey(mLine)

        if (key) {
            const value = translation.json[key] || master.json[key]
            const isLastLine = mIndex+numEmptyEndLines >= masterLength-1
            const comma = isLastLine ? "" : ","

            newTranslation.push(`  "${key}": "${value}"${comma}`)
        }
        else {
            newTranslation.push(mLine)
        }
    })

    const updatedTranslation = newTranslation.join("\n")

    if (translation.text != updatedTranslation) {
        console.log("-> Saving updated translation")
        fs.writeFileSync("./" + translation.name, updatedTranslation, {encoding: 'utf8'}, (err) => {
            if (err) console.error(`Failed to save file due to error: ${err}`)
        })
        console.log("-> Saved")
    }
    else {
        console.log("-> No changes")
    }
}

